@@ -29,6 +29,12 @@ class ObjectDict(UserDict):
         return self.data[key]
 
 
+def to_expr(var):
+    if isinstance(var, ArgoVar):
+        return var.expr
+    return str(var)
+
+
 class Expression:
     def __init__(self, expr):
         self.expr = expr
@@ -36,25 +42,25 @@ class Expression:
     def __repr__(self):
         return self.expr
 
-    def eval(self, context):
+    def eval(self, scope):
         from .dag import DAG
         from .steps import Steps
         inputs = ObjectDict()
         inputs["parameters"] = ObjectDict({
-            k: v.value for k, v in context.inputs.parameters.items()
+            k: v.value for k, v in scope.inputs.parameters.items()
         })
         steps = ObjectDict()
-        if isinstance(context, Steps):
+        if isinstance(scope, Steps):
             for step in sum([s if isinstance(s, list) else [s]
-                             for s in context], []):
+                             for s in scope], []):
                 steps[step.name] = ObjectDict({"outputs": ObjectDict({
                     "parameters": ObjectDict({k: v.value for k, v in
                                               step.outputs.parameters.items()
                                               if hasattr(v, "value")}),
                 })})
         tasks = ObjectDict()
-        if isinstance(context, DAG):
-            for task in context:
+        if isinstance(scope, DAG):
+            for task in scope:
                 tasks[task.name] = ObjectDict({"outputs": ObjectDict({
                     "parameters": ObjectDict({k: v.value for k, v in
                                               task.outputs.parameters.items()
@@ -250,7 +256,7 @@ class ArgoVar:
         return ArgoVar("asFloat(%s) >= %s" % (self.expr, other))
 
 
-class IfExpression:
+class IfExpression(ArgoVar, Expression):
     def __init__(
             self,
             _if: Union[str, ArgoVar],
@@ -265,13 +271,12 @@ class IfExpression:
             self._if = _if
         self._then = _then
         self._else = _else
-
-    def __repr__(self) -> str:
-        _then = self._then.expr if isinstance(
-            self._then, ArgoVar) else self._then
-        _else = self._else.expr if isinstance(
-            self._else, ArgoVar) else self._else
-        return "%s ? %s : %s" % (self._if, _then, _else)
+        if config["mode"] == "debug":
+            return super().__init__("(%s if %s else %s)" % (
+                to_expr(_then), to_expr(_if), to_expr(_else)))
+        else:
+            super().__init__("(%s ? %s : %s)" % (
+                self._if, to_expr(_then), to_expr(_else)))
 
 
 def if_expression(
