@@ -203,16 +203,17 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                     path="%s/inputs/artifacts/" % self.tmp_root + name,
                     optional=sign.optional, type=sign.type,
                     archive=sign.archive)
-            elif isinstance(sign, BigParameter) and config["mode"] != "debug":
+            elif isinstance(sign, BigParameter):
                 if hasattr(sign, "default"):
                     self.inputs.parameters[name] = InputParameter(
-                        save_as_artifact=True, path="%s/inputs/parameters/"
-                        % self.tmp_root + name, type=sign.type,
-                        value=sign.default)
+                        save_as_artifact=config["mode"] != "debug",
+                        path="%s/inputs/parameters/" % self.tmp_root + name,
+                        type=sign.type, value=sign.default)
                 else:
                     self.inputs.parameters[name] = InputParameter(
-                        save_as_artifact=True, path="%s/inputs/parameters/"
-                        % self.tmp_root + name, type=sign.type)
+                        save_as_artifact=config["mode"] != "debug",
+                        path="%s/inputs/parameters/" % self.tmp_root + name,
+                        type=sign.type)
             elif isinstance(sign, Parameter):
                 if hasattr(sign, "default"):
                     self.inputs.parameters[name] = InputParameter(
@@ -239,9 +240,9 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                         OutputParameter(
                         value_from_path="%s/outputs/parameters/dflow_%s_urn"
                         % (self.tmp_root, name), default="")
-            elif isinstance(sign, BigParameter) and config["mode"] != "debug":
+            elif isinstance(sign, BigParameter):
                 self.outputs.parameters[name] = OutputParameter(
-                    save_as_artifact=True,
+                    save_as_artifact=config["mode"] != "debug",
                     value_from_path="%s/outputs/parameters/" % self.tmp_root
                     + name, type=sign.type)
             elif isinstance(sign, Parameter):
@@ -427,14 +428,20 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         script += "from dflow.python.utils import handle_output_artifact," \
                   " handle_output_parameter, handle_lineage\n"
         script += f"from {op_class.__module__} import {class_name}\n\n"
-        script += "if __name__ == '__main__':\n"
         if hasattr(op_class, "func"):
-            script += "    op_obj = %s\n" % class_name
+            script += "op_obj = %s\n" % class_name
         elif op is None:
-            script += "    op_obj = %s()\n" % class_name
+            script += "op_obj = %s()\n" % class_name
         else:
-            script += "    op_obj = jsonpickle.loads(r'''%s''')\n" % \
+            script += "op_obj = jsonpickle.loads(r'''%s''')\n" % \
                 jsonpickle.dumps(op)
+        script += "def try_to_execute(input):\n"
+        script += "    try:\n"
+        script += "        return op_obj.execute(input), None\n"
+        script += "    except Exception as e:\n"
+        script += "        traceback.print_exc()\n"
+        script += "        return None, e\n\n"
+        script += "if __name__ == '__main__':\n"
         script += "    input = OPIO()\n"
         script += "    input_sign = %s.get_input_sign()\n" % class_name
         script += "    output_sign = %s.get_output_sign()\n" % class_name
@@ -492,12 +499,6 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                 script += "        input1['%s'] = list(input['%s'])[i]\n" % (
                     name, name)
             script += "        input_list.append(input1)\n"
-            script += "    def try_to_execute(input):\n"
-            script += "        try:\n"
-            script += "            return op_obj.execute(input), None\n"
-            script += "        except Exception as e:\n"
-            script += "            traceback.print_exc()\n"
-            script += "            return None, e\n"
             if self.slices.pool_size == 1:
                 script += "    output_list = []\n"
                 script += "    error_list = []\n"
