@@ -216,6 +216,10 @@ class Workflow:
             reuse_step: a list of steps to be reused in the workflow
         """
         if config["mode"] == "debug":
+            if self.context is not None:
+                assert isinstance(self.context, (Context, Executor))
+                self = self.context.render(self)
+
             if self.id is None:
                 while True:
                     self.id = self.name + "-" + randstr()
@@ -291,7 +295,7 @@ class Workflow:
             with open(os.path.join(wfdir, "status"), "w") as f:
                 f.write("Running")
             try:
-                self.entrypoint.run(self.id)
+                self.entrypoint.run(self.id, self.context)
                 with open(os.path.join(wfdir, "status"), "w") as f:
                     f.write("Succeeded")
             except Exception:
@@ -694,7 +698,7 @@ class Workflow:
                                 "local_path": os.path.abspath(os.path.join(
                                     stepdir, io, "artifacts", a)),
                             })
-                step = ArgoStep(step)
+                step = ArgoStep(step, self.id)
                 step_list.append(step)
             return step_list
 
@@ -778,7 +782,7 @@ class Workflow:
             a list of steps
         """
         workflow = self.query(fields=['metadata.name', 'status.outputs'])
-        step = ArgoStep(workflow.status)
+        step = ArgoStep(workflow.status, workflow.metadata.name)
         if hasattr(step, "outputs"):
             return step.outputs
         else:
